﻿using System.Collections.Generic;
using System.IO;
using ESET.SecureAuthentication.SDK;
using Newtonsoft.Json;

namespace SampleConsoleApplication
{
    /// <summary>
    /// Implmentation of `IUserStorage` that saves data to an in-memory dictionary that is serialised to JSON and persisted to disk.
    /// </summary>
    public class FileUserStorage : IUserStorage
    {
        private const string StoreFilename = "datastore.json";

        private readonly Dictionary<string, string> _store;

        public FileUserStorage()
        {
            // attempt to load store from disk, otherwise initialise with an empty dictionary
            _store = LoadStore() ?? new Dictionary<string, string>();
        }

        public string LoadUser(string username)
        {
            return _store.ContainsKey(username) ? _store[username] : null;
        }

        public void SaveUser(string username, string data)
        {
            // update in-memory data
            _store[username] = data;

            // persist store to disk on every save
            SaveStore(_store);
        }

        private static void SaveStore(Dictionary<string, string> store)
        {
            var json = JsonConvert.SerializeObject(store);

            using (var stream = new FileStream(StoreFilename, FileMode.Create))
            {
                using (var writer = new StreamWriter(stream))
                {
                    writer.Write(json);
                }
            }
        }

        private static Dictionary<string, string> LoadStore()
        {
            if (!File.Exists(StoreFilename))
            {
                return null;
            }

            using (var stream = new FileStream(StoreFilename, FileMode.Open))
            {
                using (var reader = new StreamReader(stream))
                {
                    var json = reader.ReadToEnd();

                    if (string.IsNullOrEmpty(json))
                    {
                        return null;
                    }

                    return JsonConvert.DeserializeObject<Dictionary<string, string>>(json);
                }
            }
        }
    }
}
