﻿using System;
using ESET.SecureAuthentication.SDK;
using ESET.SecureAuthentication.SDK.Exceptions;

namespace SampleConsoleApplication
{
    /// <summary>
    /// Sample console application for ESET Secure Authentication .NET SDK.
    /// The application will save the user information to a file on disk, preserving the state between runs.
    /// </summary>
    class Program
    {
        // SDK credentials
        private const string EsaApiKey = "";
        private const string EsaApiSecret = "";

        private static IUserStorage _userStorage;
        private static TwoFactorAuthenticator _authenticator;

        static void Main()
        {
            // initialisation
            _userStorage = new FileUserStorage();
            _authenticator = GetTwoFactorAuthenticator();

            var username = ProvisionUser();
            if (string.IsNullOrEmpty(username))
            {
                Console.WriteLine("No username was provided; exiting...");
                return;
            }

            PreAuthenticateUser(username);

            AuthenticateUser(username);

            Console.WriteLine("Press any key to exit...");
            Console.ReadKey();
        }

        /// <summary>
        /// Prompts for username and then provisions the user for two-factor authentication.
        /// </summary>
        /// <returns>Username.</returns>
        private static string ProvisionUser()
        {
            Console.WriteLine("Enter username:");
            var username = Console.ReadLine();

            if (string.IsNullOrEmpty(username))
            {              
                return null;
            }

            // `TwoFactorUser` implements `IDisposable` and must be called with "using"
            using (var twoFactorUser = GetOrCreateUser(username))
            {
                if (twoFactorUser.CredentialType != CredentialTypes.None)
                {
                    Console.WriteLine("The user is already provisioned");
                    return username; // no need to provision again
                }

                Console.WriteLine("Use Mobile Application? [y/N]:");
                var userMobileAppText = Console.ReadLine();
                var userMobileApp = !string.IsNullOrEmpty(userMobileAppText) &&
                                    userMobileAppText.Equals("y", StringComparison.InvariantCultureIgnoreCase);

                if (userMobileApp)
                {
                    var provisionResult = twoFactorUser.ProvisionMobileApp();
                    twoFactorUser.SendMobileAppProvisioningTextMessage(provisionResult);
                    Console.WriteLine("The user has been provisioned for Mobile Application OTPs");
                }
                else
                {
                    twoFactorUser.ProvisionTextMessage();
                    Console.WriteLine("The user has been provisioned for text message OTPs");
                }      
            }

            return username;
        }

        /// <summary>
        /// Attempt to get a user. If the user does not exist it will be created, prompting for the OTP.
        /// </summary>
        /// <param name="username">Name of user.</param>
        /// <returns>The user, which must be disposed of after use.</returns>
        private static TwoFactorUser GetOrCreateUser(string username)
        {
            try
            {
                return _authenticator.GetUser(username);
            }
            catch (UserNotFoundException)
            {
                Console.WriteLine("Enter mobile number for user in international format (eg \"42612345678\" where \"4\" is the country code and \"26\" is the area code):");
                var mobileNumber = Console.ReadLine();

                return _authenticator.CreateUser(username, mobileNumber);
            }
        }

        /// <summary>
        /// Pre-authenticate a user, which will return the type of credential that it expected and send a text message OTP if required.
        /// </summary>
        /// <param name="username">Name of user.</param>
        private static void PreAuthenticateUser(string username)
        {
            Console.WriteLine("Calling PreAuthenticate for user");
            
            var result = _authenticator.PreAuthenticate(username);
            
            Console.WriteLine("Expected credential: {0}", result.ExpectedCredential);
        }

        /// <summary>
        /// Prompt for an OTP which will be authenticated.
        /// </summary>
        /// <param name="username">Name of user.</param>
        private static void AuthenticateUser(string username)
        {
            // keep prompting for OTP until success or user becomes locked
            while (true)
            {
                Console.WriteLine("Enter OTP:");

                var otp = Console.ReadLine();

                try
                {
                    var result = _authenticator.Authenticate(username, otp);

                    if (result.IsAuthenticated)
                    {
                        Console.WriteLine("OTP has been authenticated");
                        break;
                    }

                    Console.WriteLine("OTP has not been authenticated");
                }
                catch (UserLockedException)
                {
                    Console.WriteLine("OTP cannot be authenticated - user has been locked");
                    break;
                }
            }
        }

        /// <summary>
        /// Create instance of "TwoFactorAuthenticator" using "UserStorage" and SDK credentials
        /// </summary>
        /// <returns></returns>
        private static TwoFactorAuthenticator GetTwoFactorAuthenticator()
        {
            var configuration = new TwoFactorConfiguration(EsaApiKey, EsaApiSecret, _userStorage)
            {
                TokenName = ".NET SDK Test",
            };

            var authenticator = new TwoFactorAuthenticator(configuration);

            return authenticator;
        }
    }
}
